using System;
using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using Telerik.Windows.Data;

namespace TreemapWpfApplication
{
    public class Folder : IDiskItem
    {
        private long size = 0;

        public Folder(string name, IEnumerable<IDiskItem> children)
        {
            this.Name = name;
            ObservableItemCollection<IDiskItem> collection = new ObservableItemCollection<IDiskItem>();
            collection.AddRange(children);
            collection.ItemChanged += new EventHandler<ItemChangedEventArgs<IDiskItem>>(collection_ItemChanged);
            collection.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(collection_CollectionChanged);
            this.Children = collection;
        }

        void collection_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            this.OnPropertyChanged("Size");
        }

        void collection_ItemChanged(object sender, ItemChangedEventArgs<IDiskItem> e)
        {
            if (e.PropertyName == "Size")
            {
                this.OnPropertyChanged("Size");
            }
        }

        public Folder(string name, IEnumerable<IDiskItem> children, long cachedSize) : this(name, children)
        {
            this.size = cachedSize;
        }

        public string Name { get; set; }

        public long Size
        {
            get
            {
                if (this.Children != null && this.Children.Count() > 0)
                    this.size = this.Children.Sum(c => c.Size);

                return this.size;
            }
        }

        public IList<IDiskItem> Children { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}